==> queueshare-0.0.55.tgz: package/src/QcRecentsPage.js <==
import AnimatedSet from './AnimatedSet.js'
import qcUnbumpPlaylist from './qcUnbumpPlaylist.js'
import qcQueueMap from './qcQueueMap.js'
import QcPlaylistMap from './QcPlaylistMap.js'
import QcPlaylistPositions from './QcPlaylistPositions.js'
import qcBumpPlaylist from './qcBumpPlaylist.js'
import QcMediaExistsAndIsTrack from './QcMediaExistsAndIsTrack.js'
import QcMediaExistsAndIsPlaylist from './QcMediaExistsAndIsPlaylist.js'
import QcsMediaExists from './QcsMediaExists.js'
import QcMountedMainOrNull from './QcMountedMainOrNull.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import qcBumpTrack from './qcBumpTrack.js'
import qcUnbumpTrack from './qcUnbumpTrack.js'
import AnimatedRefFn from './AnimatedRefFn.js'
import QcTitleBarButton from './QcTitleBarButton.js'
import QcButton from './QcButton.js'
import QcQueuePositions from './QcQueuePositions.js'
import qcsCollabMap from './qcsCollabMap.js'
import QcMediaSubtitle from './QcMediaSubtitle.js'
import QcMediaTitle from './QcMediaTitle.js'
import QcMainLiContextButton from './QcMainLiContextButton.js'
import QcMainLi from './QcMainLi.js'
import qcRecentsMap from './qcRecentsMap.js'
import QcRecentsPositions from './QcRecentsPositions.js'
import QcTitleBarForm from './QcTitleBarForm.js'
import QcTitleBar from './QcTitleBar.js'
import QcMainUl from './QcMainUl.js'
import QcMain from './QcMain.js'
import QcTitleBarInput from './QcTitleBarInput.js'
import setElChild from './setElChild.js'
import animators from './animators.js'

const MediaMainLi = (media, playlistOrNull, expandedPlaylists) => {
	const li = QcMainLi()
	li.appendChild((() => {
		const row = document.createElement('div')
		row.classList.add('mainLiRow')
		row.appendChild((() => {
			const removeButton = QcButton()
			removeButton.addEventListener('click', () => {
				if (playlistOrNull === null) {
					qcsCollabMap.delete('media-' + media + '-recentsInfo')
				} else {
					qcsCollabMap.delete('media-' + media + '-playlists-' + playlistOrNull + '-info')
				}
			})
			removeButton.appendChild(document.createTextNode('❌'))
			return removeButton
		})())
		if (QcMediaExistsAndIsPlaylist(media)) {
			const playlist = media
			row.appendChild((() => {
				const expandButton = QcButton()
				expandButton.addEventListener('click', () => {
					if (expandedPlaylists.has(playlist)) {
						expandedPlaylists.delete(playlist)
					} else {
						expandedPlaylists.add(playlist)
					}
				})
				animators.set(expandButton, () => {
					expandButton.textContent = expandedPlaylists.has(playlist) ? '➖' : '➕'
				})
				return expandButton
			})())
		}
		row.appendChild((() => {
			const contextButton = QcMainLiContextButton()
			if (QcMediaExistsAndIsTrack(media)) {
				const track = media
				contextButton.appendChild((() => {
					const queueIndexSpan = document.createElement('span')
					queueIndexSpan.classList.add('mainLiQueueIndexSpan')
					animators.set(queueIndexSpan, () => {
						const indexOrNegativeOne = qcsCollabMap.has('media-' + track + '-queueInfo') ? QcQueuePositions().indexOf(JSON.parse(qcsCollabMap.get('media-' + track + '-queueInfo')).position) : -1
						queueIndexSpan.textContent = indexOrNegativeOne === -1 ? '' : '[' + (QcQueueIsPlaying() ? (indexOrNegativeOne === 0 ? 'Playing' : indexOrNegativeOne.toString(10)) : (indexOrNegativeOne + 1).toString(10)) + '] '
					})
					return queueIndexSpan
				})())
			} else if (QcMediaExistsAndIsPlaylist(media)) {
				const playlist = media
				contextButton.appendChild((() => {
					const queueIndexSpan = document.createElement('span')
					queueIndexSpan.classList.add('mainLiQueueIndexSpan')
					animators.set(queueIndexSpan, () => {
						const firstIndexOrNegativeOne = QcQueuePositions().findIndex((position) => {
							return qcQueueMap.get(position).playlist === playlist
						})
						queueIndexSpan.textContent = firstIndexOrNegativeOne === -1 ? '' : '[' + (QcQueueIsPlaying() ? (firstIndexOrNegativeOne === 0 ? 'Playing' : firstIndexOrNegativeOne.toString(10)) : (firstIndexOrNegativeOne + 1).toString(10)) + '] '
					})
					return queueIndexSpan
				})())
			}
			contextButton.appendChild(document.createTextNode(QcMediaTitle(media)))
			const subtitle = QcMediaSubtitle(media)
			if (subtitle !== '') {
				contextButton.appendChild(document.createTextNode(' '))
				contextButton.appendChild((() => {
					const subtitleSpan = document.createElement('span')
					subtitleSpan.classList.add('mainLiSubtitleSpan')
					subtitleSpan.appendChild(document.createTextNode('(' + subtitle + ')'))
					return subtitleSpan
				})())
			}
			return contextButton
		})())
		if (QcMediaExistsAndIsTrack(media)) {
			const track = media
			row.appendChild((() => {
				const unbumpButton = QcButton()
				unbumpButton.addEventListener('click', () => {
					qcUnbumpTrack(track)
				})
				animators.set(unbumpButton, () => {
					unbumpButton.style.display = qcsCollabMap.has('media-' + track + '-queueInfo') ? '' : 'none'
				})
				unbumpButton.appendChild(document.createTextNode('⬇️'))
				return unbumpButton
			})())
		} else if (QcMediaExistsAndIsPlaylist(media)) {
			const playlist = media
			row.appendChild((() => {
				const unbumpButton = QcButton()
				unbumpButton.addEventListener('click', () => {
					qcUnbumpPlaylist(playlist)
				})
				animators.set(unbumpButton, () => {
					unbumpButton.style.display = QcQueuePositions().some((position) => {
						return qcQueueMap.get(position).playlist === playlist
					}) ? '' : 'none'
				})
				unbumpButton.appendChild(document.createTextNode('⬇️'))
				return unbumpButton
			})())
		}
		row.appendChild((() => {
			const bumpButton = QcButton()
			bumpButton.addEventListener('click', () => {
				if (QcMediaExistsAndIsTrack(media)) {
					qcBumpTrack(media)
				} else if (QcMediaExistsAndIsPlaylist(media)) {
					qcBumpPlaylist(media)
				}
			})
			bumpButton.appendChild(document.createTextNode('⬆️'))
			return bumpButton
		})())
		return row
	})())
	if (QcMediaExistsAndIsPlaylist(media)) {
		const playlist = media
		li.appendChild((() => {
			const tracksUl = document.createElement('ul')
			tracksUl.classList.add('mainLiUl')
			let mountedPositionsOrNull = null
			const innerExpandedPlaylists = new AnimatedSet()
			animators.set(tracksUl, () => {
				tracksUl.style.display = expandedPlaylists.has(playlist) ? '' : 'none'
				const positions = QcPlaylistPositions(playlist)
				if (mountedPositionsOrNull !== positions) {
					const map = QcPlaylistMap(playlist)
					tracksUl.replaceChildren(...positions.map((position) => {
						return MediaMainLi(map.get(position), playlist, innerExpandedPlaylists)
					}))
					mountedPositionsOrNull = positions
				}
			})
			return tracksUl
		})())
	}
	return li
}
const Main = (searchQueryOrNull) => {
	const main = QcMain()
	const ul = QcMainUl()
	const emptyMessageNode = document.createTextNode('empty recents message')
	const noSearchResultsMessageNode = document.createTextNode('no search results')
	const expandedPlaylists = new AnimatedSet()
	let mountedGlobalPositionsOrNull = null
	let mountedLocalMediaOrNull = null
	animators.set(main, () => {
		const globalPositions = QcRecentsPositions()
		const localMedia = mountedLocalMediaOrNull === null || QcMountedMainOrNull() !== main ? globalPositions.map((position) => {
			return qcRecentsMap.get(position)
		}) : mountedLocalMediaOrNull
		// todo this doesn't fix deleted-then-reappears-on-change-from-other-element but that's probably fine for now
		// fix this ^ (probably using something like a removedLocalMedia set) when you do a deeper refactor of this animation
		// logic which is pretty gross rn lol
		if (mountedGlobalPositionsOrNull !== globalPositions || mountedLocalMediaOrNull !== localMedia) {
			ul.replaceChildren(...localMedia.filter((media) => {
				if (!qcsCollabMap.has('media-' + media + '-recentsInfo') || !QcsMediaExists(media)) {
					return false
				}
				if (searchQueryOrNull === null) {
					return true
				}
				const searchQuery = searchQueryOrNull
				if (searchQuery === '') {
					return false
				}
				const subtitle = QcMediaSubtitle(media)
				return (QcMediaTitle(media) + (subtitle === '' ? '' : ' (' + subtitle + ')')).toUpperCase().includes(searchQuery.toUpperCase())
			}).slice(0, searchQueryOrNull === null ? 100 : 50).map((media) => {
				return MediaMainLi(media, null, expandedPlaylists)
			}))
			setElChild(main, ul.childNodes.length ? ul : (searchQueryOrNull === null ? emptyMessageNode : noSearchResultsMessageNode))
			mountedGlobalPositionsOrNull = globalPositions
			mountedLocalMediaOrNull = localMedia
		}
	})
	return main
}
const QcRecentsPage = (tabAfterTick) => {
	const SearchQuery = AnimatedRefFn('')
	const SubmittedSearchQueryOrNull = AnimatedRefFn(null)
	const page = {
		titleBar: (() => {
			const bar = QcTitleBar()
			const input = QcTitleBarInput()
			input.addEventListener('input', () => {
				SearchQuery.set(input.value)
				SubmittedSearchQueryOrNull.set(SearchQuery())
			}, {passive: true})
			animators.set(input, () => {
				input.value = SearchQuery()
			})
			bar.appendChild((() => {
				const form = QcTitleBarForm()
				form.addEventListener('submit', (event) => {
					event.preventDefault()
					input.blur()
					SubmittedSearchQueryOrNull.set(SearchQuery())
				})
				form.appendChild(input)
				return form
			})())
			bar.appendChild((() => {
				const backButton = QcTitleBarButton()
				backButton.addEventListener('click', () => {
					SearchQuery.set('')
					SubmittedSearchQueryOrNull.set(null)
				}, {passive: true})
				backButton.textContent = '⬅️'
				animators.set(backButton, () => {
					backButton.style.display = SearchQuery() === '' && SubmittedSearchQueryOrNull() === null ? 'none' : ''
				})
				return backButton
			})())
			bar.appendChild((() => {
				const clearButton = QcTitleBarButton()
				clearButton.addEventListener('click', () => {
					input.focus()
					SearchQuery.set('')
				}, {passive: true})
				clearButton.textContent = '❌'
				animators.set(clearButton, () => {
					clearButton.style.display = SearchQuery() === '' ? 'none' : ''
				})
				return clearButton
			})())
			return bar
		})(),
		main: (() => {
			const nullMain = Main(null)
			let mainOrNull = null
			let mainQueryOrNullOrUndefined
			return () => {
				const queryOrNull = SubmittedSearchQueryOrNull()
				mainOrNull = queryOrNull === null ? nullMain : (mainQueryOrNullOrUndefined === queryOrNull ? mainOrNull : Main(queryOrNull))
				mainQueryOrNullOrUndefined = queryOrNull
				return mainOrNull
			}
		})(),
	}
	return page
}

export default QcRecentsPage

==> queueshare-0.0.57.tgz: package/src/QcRecentsPage.js <==
import qcEnableLikeButton from './qcEnableLikeButton.js'
import QcMediaIsLiked from './QcMediaIsLiked.js'
import AnimatedSet from './AnimatedSet.js'
import qcUnbumpPlaylist from './qcUnbumpPlaylist.js'
import qcQueueMap from './qcQueueMap.js'
import QcPlaylistMap from './QcPlaylistMap.js'
import QcPlaylistPositions from './QcPlaylistPositions.js'
import qcBumpPlaylist from './qcBumpPlaylist.js'
import QcMediaExistsAndIsTrack from './QcMediaExistsAndIsTrack.js'
import QcMediaExistsAndIsPlaylist from './QcMediaExistsAndIsPlaylist.js'
import QcsMediaExists from './QcsMediaExists.js'
import QcMountedMainOrNull from './QcMountedMainOrNull.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import qcBumpTrack from './qcBumpTrack.js'
import qcUnbumpTrack from './qcUnbumpTrack.js'
import AnimatedRefFn from './AnimatedRefFn.js'
import QcTitleBarButton from './QcTitleBarButton.js'
import QcButton from './QcButton.js'
import QcQueuePositions from './QcQueuePositions.js'
import qcsCollabMap from './qcsCollabMap.js'
import QcMediaSubtitle from './QcMediaSubtitle.js'
import QcMediaTitle from './QcMediaTitle.js'
import QcMainLiContextButton from './QcMainLiContextButton.js'
import QcMainLi from './QcMainLi.js'
import qcRecentsMap from './qcRecentsMap.js'
import QcRecentsPositions from './QcRecentsPositions.js'
import QcTitleBarForm from './QcTitleBarForm.js'
import QcTitleBar from './QcTitleBar.js'
import QcMainUl from './QcMainUl.js'
import QcMain from './QcMain.js'
import QcTitleBarInput from './QcTitleBarInput.js'
import setElChild from './setElChild.js'
import animators from './animators.js'

const MediaMainLi = (media, playlistOrNull, expandedPlaylists) => {
	const li = QcMainLi()
	li.appendChild((() => {
		const row = document.createElement('div')
		row.classList.add('mainLiRow')
		row.appendChild((() => {
			const removeButton = QcButton()
			removeButton.addEventListener('click', () => {
				if (playlistOrNull === null) {
					qcsCollabMap.delete('media-' + media + '-recentsInfo')
				}
				else {
					qcsCollabMap.delete('media-' + media + '-playlists-' + playlistOrNull + '-info')
				}
			})
			removeButton.appendChild(document.createTextNode('❌'))
			return removeButton
		})())
		if (qcEnableLikeButton) {
			row.appendChild((() => {
				const likeButton = QcButton()
				likeButton.addEventListener('click', () => {
					qcsCollabMap.set('media-' + media + '-isLiked', JSON.stringify(!QcMediaIsLiked(media)))
				})
				animators.set(likeButton, () => {
					likeButton.textContent = QcMediaIsLiked(media) ? '🖤' : '🤍'
				})
				return likeButton
			})())
		}
		if (QcMediaExistsAndIsPlaylist(media)) {
			const playlist = media
			row.appendChild((() => {
				const expandButton = QcButton()
				expandButton.addEventListener('click', () => {
					if (expandedPlaylists.has(playlist)) {
						expandedPlaylists.delete(playlist)
					}
					else {
						expandedPlaylists.add(playlist)
					}
				})
				animators.set(expandButton, () => {
					expandButton.textContent = expandedPlaylists.has(playlist) ? '➖' : '➕'
				})
				return expandButton
			})())
		}
		row.appendChild((() => {
			const contextButton = QcMainLiContextButton()
			if (QcMediaExistsAndIsTrack(media)) {
				const track = media
				contextButton.appendChild((() => {
					const queueIndexSpan = document.createElement('span')
					queueIndexSpan.classList.add('mainLiQueueIndexSpan')
					animators.set(queueIndexSpan, () => {
						const indexOrNegativeOne = qcsCollabMap.has('media-' + track + '-queueInfo') ? QcQueuePositions().indexOf(JSON.parse(qcsCollabMap.get('media-' + track + '-queueInfo')).position) : -1
						queueIndexSpan.textContent = indexOrNegativeOne === -1 ? '' : '[' + (QcQueueIsPlaying() ? (indexOrNegativeOne === 0 ? 'Playing' : indexOrNegativeOne.toString(10)) : (indexOrNegativeOne + 1).toString(10)) + '] '
					})
					return queueIndexSpan
				})())
			}
			else if (QcMediaExistsAndIsPlaylist(media)) {
				const playlist = media
				contextButton.appendChild((() => {
					const queueIndexSpan = document.createElement('span')
					queueIndexSpan.classList.add('mainLiQueueIndexSpan')
					animators.set(queueIndexSpan, () => {
						const queuePositions = QcQueuePositions()
						const firstIndexOrNegativeOne = queuePositions.findIndex((position) => {
							return qcQueueMap.get(position).playlist === playlist
						})
						let text = ''
						if (firstIndexOrNegativeOne !== -1) {
							const firstIndex = firstIndexOrNegativeOne
							if (firstIndex === 0 && QcQueueIsPlaying()) {
								text = '[Playing] '
							}
							else {
								let lastIndex
								for (let i = queuePositions.length - 1; i >= 0; i--) {
									if (qcQueueMap.get(queuePositions[i]).playlist === playlist) {
										lastIndex = i
										break
									}
								}
								const AdjustedIndex = (index) => {
									return QcQueueIsPlaying() ? index : index + 1
								}
								text = '[' + AdjustedIndex(firstIndex) + (firstIndex === lastIndex ? '' : '-' + AdjustedIndex(lastIndex)) + '] '
							}
						}
						queueIndexSpan.textContent = text
					})
					return queueIndexSpan
				})())
			}
			contextButton.appendChild(document.createTextNode(QcMediaTitle(media)))
			const subtitle = QcMediaSubtitle(media)
			if (subtitle !== '') {
				contextButton.appendChild(document.createTextNode(' '))
				contextButton.appendChild((() => {
					const subtitleSpan = document.createElement('span')
					subtitleSpan.classList.add('mainLiSubtitleSpan')
					subtitleSpan.appendChild(document.createTextNode('(' + subtitle + ')'))
					return subtitleSpan
				})())
			}
			return contextButton
		})())
		if (QcMediaExistsAndIsTrack(media)) {
			const track = media
			row.appendChild((() => {
				const unbumpButton = QcButton()
				unbumpButton.addEventListener('click', () => {
					qcUnbumpTrack(track)
				})
				animators.set(unbumpButton, () => {
					unbumpButton.style.display = qcsCollabMap.has('media-' + track + '-queueInfo') ? '' : 'none'
				})
				unbumpButton.appendChild(document.createTextNode('🔽'))
				return unbumpButton
			})())
		}
		else if (QcMediaExistsAndIsPlaylist(media)) {
			const playlist = media
			row.appendChild((() => {
				const unbumpButton = QcButton()
				unbumpButton.addEventListener('click', () => {
					qcUnbumpPlaylist(playlist)
				})
				animators.set(unbumpButton, () => {
					unbumpButton.style.display = QcQueuePositions().some((position) => {
						return qcQueueMap.get(position).playlist === playlist
					}) ? '' : 'none'
				})
				unbumpButton.appendChild(document.createTextNode('🔽'))
				return unbumpButton
			})())
		}
		row.appendChild((() => {
			const bumpButton = QcButton()
			bumpButton.addEventListener('click', () => {
				if (QcMediaExistsAndIsTrack(media)) {
					qcBumpTrack(media)
				}
				else if (QcMediaExistsAndIsPlaylist(media)) {
					qcBumpPlaylist(media)
				}
			})
			bumpButton.appendChild(document.createTextNode('🔼'))
			return bumpButton
		})())
		return row
	})())
	if (QcMediaExistsAndIsPlaylist(media)) {
		const playlist = media
		li.appendChild((() => {
			const tracksUl = document.createElement('ul')
			tracksUl.classList.add('mainLiUl')
			let mountedPositionsOrNull = null
			const innerExpandedPlaylists = new AnimatedSet()
			animators.set(tracksUl, () => {
				tracksUl.style.display = expandedPlaylists.has(playlist) ? '' : 'none'
				const positions = QcPlaylistPositions(playlist)
				if (mountedPositionsOrNull !== positions) {
					const map = QcPlaylistMap(playlist)
					tracksUl.replaceChildren(...positions.map((position) => {
						return MediaMainLi(map.get(position), playlist, innerExpandedPlaylists)
					}))
					mountedPositionsOrNull = positions
				}
			})
			return tracksUl
		})())
	}
	return li
}
const Main = (searchQueryOrNull) => {
	const main = QcMain()
	const ul = QcMainUl()
	const emptyMessageNode = document.createTextNode('empty recents message')
	const noSearchResultsMessageNode = document.createTextNode('no search results')
	const expandedPlaylists = new AnimatedSet()
	let mountedGlobalPositionsOrNull = null
	let mountedLocalMediaOrNull = null
	animators.set(main, () => {
		const globalPositions = QcRecentsPositions()
		const localMedia = mountedLocalMediaOrNull === null || QcMountedMainOrNull() !== main ? globalPositions.map((position) => {
			return qcRecentsMap.get(position)
		}) : mountedLocalMediaOrNull
		// todo this doesn't fix deleted-then-reappears-on-change-from-other-element but that's probably fine for now
		// fix this ^ (probably using something like a removedLocalMedia set) when you do a deeper refactor of this animation
		// logic which is pretty gross rn lol
		if (mountedGlobalPositionsOrNull !== globalPositions || mountedLocalMediaOrNull !== localMedia) {
			if (searchQueryOrNull === null) {
				const ulChildren = []
				let currentTimeOrNull = null
				let currentTimeUl
				for (const media of localMedia.slice(0, 100)) {
					if (qcsCollabMap.has('media-' + media + '-recentsInfo') && QcsMediaExists(media)) {
						const date = new Date(JSON.parse(qcsCollabMap.get('media-' + media + '-recentsInfo')).time)
						date.setHours(0, 0, 0, 0)
						const time = date.getTime()
						if (currentTimeOrNull !== time) {
							currentTimeOrNull = time
							currentTimeUl = QcMainUl()
							ulChildren.push((() => {
								const li = QcMainLi()
								li.appendChild((() => {
									const row = document.createElement('div')
									row.classList.add('mainLiRow')
									row.appendChild((() => {
										const contextButton = QcMainLiContextButton()
										contextButton.classList.add('recentsDateLiContextButton')
										contextButton.textContent = date.toString().slice(4, 10)
										return contextButton
									})())
									row.style.display = 'none'// turning off date grouping for now
									return row
								})())
								li.appendChild(currentTimeUl)
								return li
							})())
						}
						currentTimeUl.appendChild(MediaMainLi(media, null, expandedPlaylists))
					}
				}
				ul.replaceChildren(...ulChildren)
				setElChild(main, ul.childNodes.length ? ul : emptyMessageNode)
			}
			else {
				const upperCaseSearchQuery = searchQueryOrNull.toUpperCase()
				ul.replaceChildren(...localMedia.filter((media) => {
					if (!qcsCollabMap.has('media-' + media + '-recentsInfo') || !QcsMediaExists(media)) {
						return false
					}
					if (upperCaseSearchQuery === '') {
						return false
					}
					const subtitle = QcMediaSubtitle(media)
					return (QcMediaTitle(media) + (subtitle === '' ? '' : ' (' + subtitle + ')')).toUpperCase().includes(upperCaseSearchQuery)
				}).slice(0, 50).map((media) => {
					return MediaMainLi(media, null, expandedPlaylists)
				}))
				setElChild(main, ul.childNodes.length ? ul : noSearchResultsMessageNode)
			}
			mountedGlobalPositionsOrNull = globalPositions
			mountedLocalMediaOrNull = localMedia
		}
	})
	return main
}
const QcRecentsPage = (tabAfterTick) => {
	const SearchQuery = AnimatedRefFn('')
	const SubmittedSearchQueryOrNull = AnimatedRefFn(null)
	const page = {
		titleBar: (() => {
			const bar = QcTitleBar()
			const input = QcTitleBarInput()
			input.addEventListener('input', () => {
				SearchQuery.set(input.value)
				SubmittedSearchQueryOrNull.set(SearchQuery())
			}, {passive: true})
			animators.set(input, () => {
				input.value = SearchQuery()
			})
			bar.appendChild((() => {
				const form = QcTitleBarForm()
				form.addEventListener('submit', (event) => {
					event.preventDefault()
					input.blur()
					SubmittedSearchQueryOrNull.set(SearchQuery())
				})
				form.appendChild(input)
				return form
			})())
			bar.appendChild((() => {
				const backButton = QcTitleBarButton()
				backButton.addEventListener('click', () => {
					SearchQuery.set('')
					SubmittedSearchQueryOrNull.set(null)
				}, {passive: true})
				backButton.textContent = '⬅️'
				animators.set(backButton, () => {
					backButton.style.display = SearchQuery() === '' && SubmittedSearchQueryOrNull() === null ? 'none' : ''
				})
				return backButton
			})())
			bar.appendChild((() => {
				const clearButton = QcTitleBarButton()
				clearButton.addEventListener('click', () => {
					input.focus()
					SearchQuery.set('')
				}, {passive: true})
				clearButton.textContent = '❌'
				animators.set(clearButton, () => {
					clearButton.style.display = SearchQuery() === '' ? 'none' : ''
				})
				return clearButton
			})())
			return bar
		})(),
		main: (() => {
			const nullMain = Main(null)
			let mainOrNull = null
			let mainQueryOrNullOrUndefined
			return () => {
				const queryOrNull = SubmittedSearchQueryOrNull()
				mainOrNull = queryOrNull === null ? nullMain : (mainQueryOrNullOrUndefined === queryOrNull ? mainOrNull : Main(queryOrNull))
				mainQueryOrNullOrUndefined = queryOrNull
				return mainOrNull
			}
		})(),
	}
	return page
}

export default QcRecentsPage
